/*
    SPDX-FileCopyrightText: 2014 Aleix Pol Gonzalez <aleixpol@blue-systems.com>

    SPDX-License-Identifier: LGPL-2.1-or-later
*/

#include <purpose/pluginbase.h>
#include "youtubejobcomposite.h"
#include "youtubejob.h"

#include <KPluginFactory>

class YoutubePlugin : public Purpose::PluginBase
{
Q_OBJECT
public:
    YoutubePlugin(QObject* parent, const QVariantList& args)
        : Purpose::PluginBase(parent)
    {
        Q_UNUSED(args);
    }

    Purpose::Job* createJob() const override
    {
        return new YoutubeJobComposite;
    }
};

K_PLUGIN_CLASS_WITH_JSON(YoutubePlugin, "youtubeplugin.json")

EXPORT_SHARE_VERSION

#include "youtubeplugin.moc"
